@@ -1,4 +1,7 @@
-# RT::Client::REST
+#!perl
+# vim: softtabstop=4 tabstop=4 shiftwidth=4 ft=perl expandtab smarttab
+# PODNAME: RT::Client::REST
+# ABSTRACT: Client for RT using REST API
 #
 # Dmitri Tikhonov <dtikhonov@yahoo.com>
 #
@@ -18,25 +21,21 @@
 #------------------------
 
 
-package RT::Client::REST;
-
 use strict;
 use warnings;
 
-use vars qw/$VERSION/;
-$VERSION = '0.50';
-$VERSION = eval $VERSION;
-
-use Error qw(:try);
+package RT::Client::REST;
+$RT::Client::REST::VERSION = '0.72';
+use Try::Tiny;
 use HTTP::Cookies;
 use HTTP::Request::Common;
-use RT::Client::REST::Exception 0.18;
+use RT::Client::REST::Exception;
 use RT::Client::REST::Forms;
 use RT::Client::REST::HTTPClient;
 
 # Generate accessors/mutators
-for my $method (qw(server _cookie timeout)) {
-    no strict 'refs';
+for my $method (qw(server _cookie timeout verbose_errors user_agent_args)) {
+    no strict 'refs'; ## no critic (ProhibitNoStrict)
     *{__PACKAGE__ . '::' . $method} = sub {
         my $self = shift;
         if (@_) {
@@ -92,16 +91,23 @@ sub login {
     # server-side errors we bubble up and ignore all others.
     try {
         $self->_cookie(undef);  # Start a new session.
-        $self->_submit("ticket/1", undef, \%opts);
-    } catch RT::Client::REST::AuthenticationFailureException with {
-        shift->rethrow;
-    } catch RT::Client::REST::MalformedRTResponseException with {
-        shift->rethrow;
-    } catch RT::Client::REST::RequestTimedOutException with {
-        shift->rethrow;
-    } catch RT::Client::REST::HTTPException with {
-        shift->rethrow;
-    } catch Exception::Class::Base with {
+        $self->_submit('ticket/1', undef, \%opts);
+    }
+    catch {
+        die $_ unless blessed $_ && $_->can('rethrow');
+
+        my $err = $_;
+        if (grep { $err->isa($_) } (
+                'RT::Client::REST::AuthenticationFailureException',
+                'RT::Client::REST::MalformedRTResponseException',
+                'RT::Client::REST::RequestTimedOutException',
+                'RT::Client::REST::HTTPException',
+            )) {
+            shift->rethrow
+        }
+        if (! $err->isa('Exception::Class::Base')) {
+            die $err
+        }
         # ignore others.
     };
 }
@@ -124,7 +130,7 @@ sub show {
     }
 
     my $form = form_parse($self->_submit("$type/$id")->decoded_content);
-    my ($c, $o, $k, $e) = @{$$form[0]};
+    my ($c, $o, $k) = @{$$form[0]}; # my ($c, $o, $k, $e)
 
     if (!@$o && $c) {
         RT::Client::REST::Exception->_rt_content_to_exception($c)->throw;
@@ -146,13 +152,38 @@ sub get_attachment_ids {
     my $form = form_parse(
         $self->_submit("$type/$id/attachments/")->decoded_content
     );
-    my ($c, $o, $k, $e) = @{$$form[0]};
+    my ($c, $o, $k) = @{$$form[0]}; # my ($c, $o, $k, $e)
 
     if (!@$o && $c) {
         RT::Client::REST::Exception->_rt_content_to_exception($c)->throw;
     }
 
-    return $k->{Attachments} =~ m/(\d+):/mg;
+    return $k->{Attachments} =~ m/^\s*(\d+):/mg;
+}
+
+sub get_attachments_metadata {
+    my $self = shift;
+
+    $self->_assert_even(@_);
+
+    my %opts = @_;
+
+    my $type = $self->_valid_type(delete($opts{type}) || 'ticket');
+    my $id = $self->_valid_numeric_object_id(delete($opts{id}));
+
+    my $form = form_parse(
+        $self->_submit("$type/$id/attachments/")->decoded_content
+    );
+    my ($c, $o, $k) = @{$$form[0]}; # my ($c, $o, $k, $e)
+
+    if (!@$o && $c) {
+        RT::Client::REST::Exception->_rt_content_to_exception($c)->throw;
+    }
+    return map {
+      # Matches: '50008989: (Unnamed) (text/plain / 1.9k),'
+      my @c = $_ =~ m/^\s*(\d+):\s+(.+)\s+\(([^\s]+)\s+\/\s+([^\s]+)\)\s*,?\s*$/;
+      { id => $c[0], Filename => ( defined($c[1]) && ( $c[1] eq '(Unnamed)' ) ) ? undef : $c[1], Type => $c[2], Size => $c[3] };
+    } split(/\n/, $k->{Attachments});
 }
 
 sub get_attachment {
@@ -176,7 +207,7 @@ sub get_attachment {
     }
     my $form = form_parse($content);
 
-    my ($c, $o, $k, $e) = @{$$form[0]};
+    my ($c, $o, $k) = @{$$form[0]}; # my ($c, $o, $k, $e)
 
     if (!@$o && $c) {
         RT::Client::REST::Exception->_rt_content_to_exception($c)->throw;
@@ -198,21 +229,21 @@ sub get_links {
     my $form = form_parse(
         $self->_submit("$type/$id/links/$id")->decoded_content
     );
-    my ($c, $o, $k, $e) = @{$$form[0]};
+    my ($c, $o, $k) = @{$$form[0]}; # my ($c, $o, $k, $e)
 
     if (!@$o && $c) {
         RT::Client::REST::Exception->_rt_content_to_exception($c)->throw;
     }
 
     # Turn the links into id lists
-    foreach my $key (keys(%$k)) {
+    for my $key (keys(%$k)) {
         try {
             $self->_valid_link_type($key);
             my @list = split(/\s*,\s*/,$k->{$key});
             #use Data::Dumper;
             #print STDERR Dumper(\@list);
             my @newlist = ();
-            foreach my $val (@list) {
+            for my $val (@list) {
                if ($val =~ /^fsck\.com-\w+\:\/\/(.*?)\/(.*?)\/(\d+)$/) {
                    # We just want the ids, not the URI
                    push(@newlist, {'type' => $2, 'instance' => $1, 'id' => $3 });
@@ -224,7 +255,13 @@ sub get_links {
             # Copy the newly created list
             $k->{$key} = ();
             $k->{$key} = \@newlist;
-        } catch RT::Client::REST::InvalidParameterValueException with {
+        }
+        catch {
+            die $_ unless blessed $_ && $_->can('rethrow');
+
+            if (! $_->isa('RT::Client::REST::InvalidParameterValueException')) {
+                $_->rethrow;
+            }
             # Skip it because the keys are not always valid e.g., 'id'
         }
     }
@@ -250,7 +287,7 @@ sub get_transaction_ids {
     } elsif ('ARRAY' eq ref($tr_type)) {
         # OK, more than one type.  Call ourselves for each.
         # NOTE: this may be very expensive.
-        return sort map {
+        my @return = sort map {
             $self->get_transaction_ids(
                 parent_id => $parent_id,
                 transaction_type => $_,
@@ -260,6 +297,7 @@ sub get_transaction_ids {
             # error this way.
             $self->_valid_transaction_type($_)
         } @$tr_type;
+        return @return
     } else {
         $tr_type = $self->_valid_transaction_type($tr_type);
         $path = "$type/$parent_id/history/type/$tr_type"
@@ -294,7 +332,7 @@ sub get_transaction {
     my $form = form_parse(
         $self->_submit("$type/$parent_id/history/id/$id")->decoded_content
     );
-    my ($c, $o, $k, $e) = @{$$form[0]};
+    my ($c, $o, $k) = @{$$form[0]}; # my ($c, $o, $k, $e)
 
     if (!@$o && $c) {
         RT::Client::REST::Exception->_rt_content_to_exception($c)->throw;
@@ -313,12 +351,26 @@ sub search {
     my $type = $self->_valid_type(delete($opts{type}));
     my $query = delete($opts{query});
     my $orderby = delete($opts{orderby});
+    my $format = delete($opts{format});
+    if (defined($format)) {
+        $format = undef if $format ne 's'
+    }
 
     my $r = $self->_submit("search/$type", {
         query => $query,
         (defined($orderby) ? (orderby => $orderby) : ()),
+        (defined($format) ? (format => $format) : ()),
     });
 
+    if (defined($format) and $format eq 's') {
+        my @results;
+        # while() never stops if the method is used in the regex
+        my $text = $r->decoded_content;
+        while ($text =~ m/^(\d+): (.*)/gm) {
+            push @results, [$1, $2]
+        }
+        return @results
+    }
     return $r->decoded_content =~ m/^(\d+):/gm;
 }
 
@@ -380,20 +432,28 @@ sub comment {
     my $ticket_id = $self->_valid_numeric_object_id(delete($opts{ticket_id}));
     my $msg = $self->_valid_comment_message(delete($opts{message}));
 
-    my @objects = ("Ticket", "Action", "Text");
+    my @objects = ('Ticket', 'Action', 'Text');
     my %values  = (
         Ticket      => $ticket_id,
         Action      => $action,
         Text        => $msg,
     );
 
+    if (exists($opts{html})) {
+        if ($opts{html}) {
+            push @objects, 'Content-Type';
+            $values{'Content-Type'} = 'text/html';
+        }
+        delete($opts{html});
+    }
+
     if (exists($opts{cc})) {
-        push @objects, "Cc";
+        push @objects, 'Cc';
         $values{Cc} = delete($opts{cc});
     }
 
     if (exists($opts{bcc})) {
-        push @objects, "Bcc";
+        push @objects, 'Bcc';
         $values{Bcc} = delete($opts{bcc});
     }
 
@@ -405,7 +465,7 @@ sub comment {
                 "'attachments' must be an array reference",
             );
         }
-        push @objects, "Attachment";
+        push @objects, 'Attachment';
         $values{Attachment} = $files;
 
         for (my $i = 0; $i < @$files; ++$i) {
@@ -416,7 +476,7 @@ sub comment {
             }
 
             my $index = $i + 1;
-            $data{"attachment_$index"} = bless([ $files->[$i] ], "Attachment");
+            $data{"attachment_$index"} = bless([ $files->[$i] ], 'Attachment');
         }
     }
 
@@ -440,7 +500,7 @@ sub merge_tickets {
     return;
 }
 
-sub link {
+sub _link {
     my $self = shift;
     $self->_assert_even(@_);
     my %opts = @_;
@@ -464,10 +524,10 @@ sub link {
     return;
 }
 
-sub link_tickets { shift->link(@_, type => 'ticket') }
+sub link_tickets { shift->_link(@_, type => 'ticket') }
 
-sub unlink { shift->link(@_, unlink => 1) }
-sub unlink_tickets { shift->link(@_, type => 'ticket', unlink => 1) }
+# sub unlink { shift->_link(@_, unlink => 1) } ## nothing calls this & undocumented, so commenting out for now
+sub unlink_tickets { shift->_link(@_, type => 'ticket', unlink => 1) }
 
 sub _ticket_action {
     my $self = shift;
@@ -505,13 +565,13 @@ sub _submit {
         unless (ref $content) {
             # If it's just a string, make sure LWP handles it properly.
             # (By pretending that it's a file!)
-            $content = [ content => [undef, "", Content => $content] ];
+            $content = [ content => [undef, q(), Content => $content] ];
         }
         elsif (ref $content eq 'HASH') {
             my @data;
-            foreach my $k (keys %$content) {
+            for my $k (keys %$content) {
                 if (ref $content->{$k} eq 'ARRAY') {
-                    foreach my $v (@{ $content->{$k} }) {
+                    for my $v (@{ $content->{$k} }) {
                         push @data, $k, $v;
                     }
                 }
@@ -526,7 +586,7 @@ sub _submit {
     unless ($self->_cookie || $self->basic_auth_cb) {
         unless (defined($auth)) {
             RT::Client::REST::RequiredAttributeUnsetException->throw(
-                "You must log in first",
+                'You must log in first',
             );
         }
         push @$data, %$auth;
@@ -548,9 +608,9 @@ sub _submit {
     }
 
     # Then we send the request and parse the response.
-    $self->logger->debug("request: ", $req->as_string);
+    $self->logger->debug('request: ', $req->as_string);
     my $res = $self->_ua->request($req);
-    $self->logger->debug("response: ", $res->as_string);
+    $self->logger->debug('response: ', $res->as_string);
 
     if ($res->is_success) {
         # The content of the response we get from the RT server consists
@@ -558,14 +618,17 @@ sub _submit {
         # a blank line, and arbitrary text.
 
         my ($head, $text) = split /\n\n/, $res->decoded_content(charset => 'none'), 2;
-        my ($status, @headers) = split /\n/, $head;
-        $text =~ s/\n*$/\n/ if ($text);
+        my ($status) = split /\n/, $head; # my ($status, @headers) = split /\n/, $head;
 
+        # Example:
         # "RT/3.0.1 401 Credentials required"
-	if ($status !~ m#^RT/\d+(?:\S+) (\d+) ([\w\s]+)$#) {
-            RT::Client::REST::MalformedRTResponseException->throw(
-                "Malformed RT response received from " . $self->server,
-            );
+        if ($status !~ m#^RT/\d+(?:\S+) (\d+) ([\w\s]+)$#) {
+            my $err_msg = 'Malformed RT response received from ' . $self->server;
+            if ($self->verbose_errors) {
+                $err_msg = "Malformed RT response received from " . $self->_uri($uri) .
+                  " with this response: " . substr($text || '', 0, 200) . '....';
+            }
+            RT::Client::REST::MalformedRTResponseException->throw($err_msg);
         }
 
         # Our caller can pretend that the server returned a custom HTTP
@@ -589,21 +652,28 @@ sub _submit {
                 if (exists $d{user}) {
                     RT::Client::REST::AuthenticationFailureException->throw(
                         code    => $res->code,
-                        message => "Incorrect username or password",
+                        message => 'Incorrect username or password',
                     );
                 }
-                elsif ($req->header("Cookie")) {
+                elsif ($req->header('Cookie')) {
                     # We'll retry the request with credentials, unless
                     # we only wanted to logout in the first place.
                     #$session->delete;
                     #return submit(@_) unless $uri eq "$REST/logout";
                 }
-            } else {
+                else {
+                    RT::Client::REST::AuthenticationFailureException->throw(
+                        code    => $res->code,
+                        message => 'Server said: '. $res->message,
+                    );
+                }
+            }
+            else {
                 RT::Client::REST::Exception->_rt_content_to_exception(
                     $res->decoded_content)
                 ->throw(
                     code    => $res->code,
-                    message => "RT server returned this error: " .
+                    message => 'RT server returned this error: ' .
                                $res->decoded_content,
                 );
             }
@@ -612,10 +682,10 @@ sub _submit {
         500 == $res->code &&
         # Older versions of HTTP::Response populate 'message', newer
         # versions populate 'content'.  This catches both cases.
-        ($res->decoded_content || $res->message) =~ /read timeout/
+        ($res->decoded_content || $res->message) =~ m/read timeout/
     ) {
         RT::Client::REST::RequestTimedOutException->throw(
-            "Your request to " . $self->server . " timed out",
+            'Your request to ' . $self->server . ' timed out',
         );
     } elsif (302 == $res->code && !$self->{'_redirected'}) {
         $self->{'_redirected'} = 1;     # We only allow one redirection
@@ -629,9 +699,13 @@ sub _submit {
         $self->server($new_server);
         return $self->_submit($uri, $content, $auth);
     } else {
+        my $err_msg = $res->message;
+        if ($self->verbose_errors) {
+            $err_msg = $res->message . ' fetching ' . $self->_uri($uri);
+        };
         RT::Client::REST::HTTPException->throw(
             code    => $res->code,
-            message => $res->message,
+            message => $err_msg,
         );
     }
 
@@ -642,9 +716,14 @@ sub _ua {
     my $self = shift;
 
     unless (exists($self->{_ua})) {
+
+        my $args = $self->user_agent_args || {};
+        die "user_agent_args must be a hashref" unless ref($args) eq 'HASH';
         $self->{_ua} = RT::Client::REST::HTTPClient->new(
             agent => $self->_ua_string,
             env_proxy => 1,
+            max_redirect => 1,
+            %$args,
         );
         if ($self->timeout) {
             $self->{_ua}->timeout($self->timeout);
@@ -657,6 +736,11 @@ sub _ua {
     return $self->{_ua};
 }
 
+sub user_agent {
+    shift->_ua;
+}
+
+
 sub basic_auth_cb {
     my $self = shift;
 
@@ -673,6 +757,8 @@ sub basic_auth_cb {
     return $self->{_basic_auth_cb};
 }
 
+# Sometimes PodCoverageTests think LOGGER_METHODS is a vanilla sub
+
 use constant LOGGER_METHODS => (qw(debug warn info error));
 
 sub logger {
@@ -780,7 +866,7 @@ sub _valid_transaction_type {
     unless (grep { $type eq $_ } $self->_list_of_valid_transaction_types) {
         RT::Client::REST::InvalidParameterValueException->throw(
             "'$type' is not a valid transaction type.  Allowed types: " .
-            join(", ", $self->_list_of_valid_transaction_types)
+            join(', ', $self->_list_of_valid_transaction_types)
         );
     }
 
@@ -810,10 +896,10 @@ sub _uri { shift->_rest . '/' . shift }
 
 sub _ua_string {
     my $self = shift;
-    return ref($self) . '/' . $self->_version;
+    return ref($self) . '/' . ($self->_version || '???');
 }
 
-sub _version { $VERSION }
+sub _version { $RT::Client::REST::VERSION }
 
 {
     # This is a noop logger: it discards all log messages.  It is the default
@@ -841,9 +927,10 @@ sub _version { $VERSION }
     # The problem with the second approach is that it creates unrelated
     # methods in RT::Client::REST namespace.
     package RT::Client::REST::NoopLogger;
-    sub new { bless \(my $logger) }
+$RT::Client::REST::NoopLogger::VERSION = '0.72';
+sub new { bless \(my $logger), __PACKAGE__ }
     for my $method (RT::Client::REST::LOGGER_METHODS) {
-        no strict 'refs';
+        no strict 'refs'; ## no critic (ProhibitNoStrict)
         *{$method} = sub {};
     }
 }
@@ -854,13 +941,19 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
-RT::Client::REST -- talk to RT installation using REST protocol.
+RT::Client::REST - Client for RT using REST API
+
+=head1 VERSION
+
+version 0.72
 
 =head1 SYNOPSIS
 
-  use Error qw(:try);
+  use Try::Tiny;
   use RT::Client::REST;
 
   my $rt = RT::Client::REST->new(
@@ -870,17 +963,24 @@ RT::Client::REST -- talk to RT installation using REST protocol.
 
   try {
     $rt->login(username => $user, password => $pass);
-  } catch Exception::Class::Base with {
-    die "problem logging in: ", shift->message;
+  }
+  catch {
+    if ($_->isa('Exception::Class::Base') {
+      die "problem logging in: ", shift->message;
+    }
   };
 
   try {
     # Get ticket #10
     $ticket = $rt->show(type => 'ticket', id => 10);
-  } catch RT::Client::REST::UnauthorizedActionException with {
-    print "You are not authorized to view ticket #10\n";
-  } catch RT::Client::REST::Exception with {
-    # something went wrong.
+  }
+  catch {
+    if ($_->isa('RT::Client::REST::UnauthorizedActionException')) {
+      print "You are not authorized to view ticket #10\n";
+    }
+    if ($_->isa('RT::Client::REST::Exception')) {
+      # something went wrong.
+    }
   };
 
 =head1 DESCRIPTION
@@ -890,6 +990,8 @@ to implement some RT interactions from my application, but did not feel that
 invoking a shell command is appropriate.  Thus, I took B<rt> tool, written
 by Abhijit Menon-Sen, and converted it to an object-oriented Perl module.
 
+=for Pod::Coverage LOGGER_METHODS
+
 =head1 USAGE NOTES
 
 This API mimics that of 'rt'.  For a more OO-style APIs, please use
@@ -936,12 +1038,37 @@ returns username and password:
     return ($username, $password);
   }
 
+=item B<user_agent_args>
+
+A hashref which will be passed to the user agent's constructor for
+maximum flexibility.
+
+=item B<user_agent>
+
+Accessor to the user_agent object.
+
 =item B<logger>
 
 A logger object.  It should be able to debug(), info(), warn() and
 error().  It is not widely used in the code (yet), and so it is
 mostly useful for development.
 
+Something like this will get you started:
+
+  use Log::Dispatch;
+  my $log = Log::Dispatch->new(
+    outputs => [ [ 'Screen', min_level => 'debug' ] ],
+  );
+  my $rt = RT::Client::REST->new(
+    server => ... etc ...
+    logger => $log
+  );
+
+=item B<verbose_errors>
+
+On user-agent errors, report some more information about what is going
+wrong. Defaults are pretty laconic about the "Malformed RT response".
+
 =back
 
 =item login (username => 'root', password => 'password')
@@ -950,7 +1077,7 @@ mostly useful for development.
 Log in to RT.  Throws an exception on error.
 
 Usually, if the other side uses basic HTTP authentication, you do not
-have to log in, but rather prodive HTTP username and password instead.
+have to log in, but rather provide HTTP username and password instead.
 See B<basic_auth_cb> above.
 
 =item show (type => $type, id => $id)
@@ -972,7 +1099,7 @@ text of the ticket.
 Returns numeric ID of the new object.  If numeric ID cannot be parsed from
 the response, B<RT::Client::REST::MalformedRTResponseException> is thrown.
 
-=item search (type => $type, query => $query, %opts)
+=item search (type => $type, query => $query, format => $format, %opts)
 
 Search for object of type C<$type> by using query C<$query>.  For
 example:
@@ -985,6 +1112,8 @@ example:
 
 C<%opts> is a list of key-value pairs:
 
+=for stopwords orderby
+
 =over 4
 
 =item B<orderby>
@@ -1002,7 +1131,7 @@ order (minus).  For example:
 
 =back
 
-C<search> returns the list of numeric IDs of objects that matched
+By default, C<search> returns the list of numeric IDs of objects that matched
 your query.  You can then use these to retrieve object information
 using C<show()> method:
 
@@ -1012,15 +1141,45 @@ using C<show()> method:
   );
   for my $id (@ids) {
     my ($ticket) = $rt->show(type => 'ticket', id => $id);
-    print "Subject: ", $ticket->{Subject}, "\n";
+    say "Subject: ", $ticket->{Subject}
+  }
+
+C<search> can return a list of lists of ID and Subject when asked for format 's'.
+
+  my @results = $rt->search(
+    type => 'ticket',
+    query => "Status = 'stalled'",
+    format => 's',
+  );
+  for my $result (@results) {
+    say "ID: $result[0], Subject: $result[1]"
   }
 
 =item comment (ticket_id => $id, message => $message, %opts)
 
+=for stopwords bcc
+
 Comment on a ticket with ID B<$id>.
-Optionally takes arguments B<cc> and B<bcc> which are references to lists
-of e-mail addresses and B<attachments> which is a list of filenames to
-be attached to the ticket.
+
+Optionally takes arguments:
+
+=over 2
+
+=item B<cc> and B<bcc>
+
+References to lists of e-mail addresses
+
+=item B<attachments>
+
+A list of filenames to be attached to the ticket
+
+=for stopwords html
+
+=item B<html>
+
+When true, indicates to RT that the message is html
+
+=back
 
   $rt->comment(
     ticket_id   => 5,
@@ -1028,6 +1187,12 @@ be attached to the ticket.
     cc          => [qw(dmitri@localhost some@otherdude.com)],
   );
 
+  $rt->comment(
+    ticket_id   => 5,
+    message     => "<b>Wild thing</b>, you make my <i>heart sing</i>",
+    html        => 1
+  );
+
 =item correspond (ticket_id => $id, message => $message, %opts)
 
 Add correspondence to ticket ID B<$id>.  Takes optional B<cc>,
@@ -1037,6 +1202,20 @@ B<bcc>, and B<attachments> parameters (see C<comment> above).
 
 Get a list of numeric attachment IDs associated with ticket C<$id>.
 
+=for stopwords undecoded
+
+=item get_attachments_metadata (id => $id)
+
+Get a list of the metadata related to every attachment of the ticket <$id>
+Every member of the list is a hashref with the shape:
+
+  {
+    id       => $attachment_id,
+    Filename => $attachment_filename,
+    Type     => $attachment_type,
+    Size     => $attachment_size,
+  }
+
 =item get_attachment (parent_id => $parent_id, id => $id, undecoded => $bool)
 
 Returns reference to a hash with key-value pair describing attachment
@@ -1061,7 +1240,7 @@ have the following options:
 
 =item B<type>
 
-Type of the object transactions are associated wtih.  Defaults to "ticket"
+Type of the object transactions are associated with.  Defaults to "ticket"
 (I do not think server-side supports anything else).  This is designed with
 the eye on the future, as transactions are not just for tickets, but for
 other objects as well.
@@ -1074,6 +1253,8 @@ more than one type, pass an array reference.
 
 Transactions may be of the following types (case-sensitive):
 
+=for stopwords AddLink AddWatcher CustomField DelWatcher DeleteLink DependedOnBy DependsOn EmailRecord HasMember MemberOf ReferredToBy RefersTo
+
 =over 2
 
 =item AddLink
@@ -1116,6 +1297,8 @@ Get a hashref representation of transaction C<$id> associated with
 parent object C<$id>.  You can optionally specify parent object type in
 C<%opts> (defaults to 'ticket').
 
+=for stopwords dst src
+
 =item merge_tickets (src => $id1, dst => $id2)
 
 Merge ticket B<$id1> into ticket B<$id2>.
@@ -1162,6 +1345,8 @@ Take ticket C<$id>.
 This will throw C<RT::Client::REST::AlreadyTicketOwnerException> if you are
 already the ticket owner.
 
+=for stopwords Untake untake
+
 =item untake (id => $id)
 
 Untake ticket C<$id>.
@@ -1179,9 +1364,8 @@ already the ticket owner.
 =head1 EXCEPTIONS
 
 When an error occurs, this module will throw exceptions.  I recommend
-using Error.pm's B<try{}> mechanism to catch them, but you may also use
-simple B<eval{}>.  The former will give you flexibility to catch just the
-exceptions you want.
+using L<Try::Tiny> or L<Syntax::Keyword::Try> B<try{}> mechanism to catch them,
+but you may also use simple B<eval{}>.
 
 Please see L<RT::Client::REST::Exception> for the full listing and
 description of all the exceptions.
@@ -1202,10 +1386,6 @@ The following modules are required:
 
 =item
 
-Error
-
-=item
-
 Exception::Class
 
 =item
@@ -1233,23 +1413,80 @@ Most likely.  Please report.
 
 =head1 VARIOUS NOTES
 
+=for stopwords TODO
+
 B<RT::Client::REST> does not (at the moment, see TODO file) retrieve forms from
 RT server, which is either good or bad, depending how you look at it.
 
-=head1 VERSION
+=head1 AUTHOR
 
-This is version 0.50 of B<RT::Client::REST>.
+Dean Hamstead <dean@fragfest.com.au>
 
-=head1 AUTHORS
+=head1 COPYRIGHT AND LICENSE
 
-Original /usr/bin/rt was written by Abhijit Menon-Sen <ams@wiw.org>.  rt
-was later converted to this module by Dmitri Tikhonov <dtikhonov@yahoo.com>.
-In January of 2008, Damien "dams" Krotkine <dams@cpan.org> joined as the
-project's co-maintainer. JLMARTIN has become co-maintainer as of March 2010.
-SRVSH became a co-maintainer in November 2015.
+This software is copyright (c) 2023, 2020 by Dmitri Tikhonov.
 
-=head1 LICENSE
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
-Perl license.
+=head1 CONTRIBUTORS
+
+=for stopwords Abhijit Menon-Sen belg4mit bobtfish Byron Ellacott Dean Hamstead DJ Stauffer dkrotkine Dmitri Tikhonov Marco Pessotto pplusdomain Sarvesh D Søren Lund Tom Harrison
+
+=over 4
+
+=item *
+
+Abhijit Menon-Sen <ams@wiw.org>
+
+=item *
+
+belg4mit <belg4mit>
+
+=item *
+
+bobtfish <bobtfish@bobtfish.net>
+
+=item *
+
+Byron Ellacott <code@bje.id.au>
+
+=item *
+
+Dean Hamstead <djzort@cpan.org>
+
+=item *
+
+DJ Stauffer <dj@djstauffer.com>
+
+=item *
+
+dkrotkine <dkrotkine@gmail.com>
+
+=item *
+
+Dmitri Tikhonov <dmitri@cpan.org>
+
+=item *
+
+Marco Pessotto <melmothx@gmail.com>
+
+=item *
+
+pplusdomain <pplusdomain@gmail.com>
+
+=item *
+
+Sarvesh D <sarveshd@openmailbox.org>
+
+=item *
+
+Søren Lund <soren@lund.org>
+
+=item *
+
+Tom Harrison <tomh@apnic.net>
+
+=back
 
 =cut
